# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Custom tab widget with custom tabbar."""

# yapf: disable

# Standard library imports
import sys

# Third party imports
from qtpy.QtCore import Qt, QTimer, Signal
from qtpy.QtGui import QIcon
from qtpy.QtWidgets import QHBoxLayout, QLabel, QSizePolicy, QStackedWidget, QVBoxLayout, QWidget

# Local imports
from anaconda_navigator.utils.ads import AdvertisementsList
from anaconda_navigator.widgets import (
    ButtonLink, ButtonToolBase, FrameBase, FrameTabBar, FrameTabBody, LabelBase, LabelImageLinkVertical, StackBody,
)


# yapf: enable


class LabelTabHeader(LabelBase):
    """Label used in CSS styling."""


class FrameTabBarBottom(FrameBase):
    """Frame used in CSS styling."""


class FrameTabBarLink(FrameBase):
    """Frame used in CSS styling."""


class FrameTabBarSocial(FrameBase):
    """Frame used in CSS styling."""


class ButtonTab(ButtonToolBase):
    """Button used in custom tab bar for CSS styling."""
    def __init__(self, *args, **kwargs):
        """Button used in custom tab bar for CSS styling."""
        super(ButtonTab, self).__init__(*args, **kwargs)
        self.setCheckable(True)
        self.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)

    def keyPressEvent(self, event):
        """Override Qt method."""
        key = event.key()
        if key in [Qt.Key_Enter, Qt.Key_Return]:
            self.animateClick()
        super(ButtonTab, self).keyPressEvent(event)


class TabBar(QWidget):
    """Custom QTabBar that includes centered icons and text bellow the icon."""

    sig_index_changed = Signal(int)
    sig_url_clicked = Signal(object)
    sig_update_banner = Signal(object)

    def __init__(self, *args, **kwargs):
        """Custom QTabBar."""
        super(TabBar, self).__init__(*args, **kwargs)
        self.buttons = []
        self.links = []
        self.links_social = []
        self.frame_bottom = FrameTabBarBottom()
        self.frame_social = FrameTabBarSocial()
        self.frame_link = FrameTabBarLink()
        self.current_index = None

        # Layouts
        self.layout_top = QVBoxLayout()
        self.layout_link = QVBoxLayout()
        self.banners_stack = QStackedWidget()
        self.layout_social = QHBoxLayout()
        self._label_links_header = LabelTabHeader('')

        layout = QVBoxLayout()
        layout.addLayout(self.layout_top)
        layout.addStretch()

        self.frame_link.setLayout(self.layout_link)
        self.frame_social.setLayout(self.layout_social)

        layout_bottom = QVBoxLayout()
        layout_bottom.addWidget(self.frame_link)
        layout_bottom.addWidget(self.frame_social)
        self.frame_bottom.setLayout(layout_bottom)

        layout.addWidget(self.frame_bottom)
        #        self.layout_bottom.addWidget(self._label_links_header, 0,
        #                                     Qt.AlignLeft)
        self.setLayout(layout)

    def set_links_header(self, text):
        """Add links header to the bottom of the custom tab bar."""
        self._label_links_header.setText(text)

    def add_social(self, text, url=None):
        """Add social link on bottom of side bar."""
        button = ButtonLink()
        button.setText(' ')
        button.setObjectName(text.lower())
        button.setFocusPolicy(Qt.StrongFocus)
        button.clicked.connect(lambda v=None, url=url: self.sig_url_clicked.emit(url))
        self.layout_social.addWidget(button, 0, Qt.AlignCenter)
        self.links_social.append(button)

    def add_link(self, text, url=None):
        """Add link on bottom of side bar."""
        button = ButtonLink()
        button.setText(text)
        button.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        button.setFocusPolicy(Qt.StrongFocus)
        button.clicked.connect(lambda v=None, url=url: self.sig_url_clicked.emit(url))
        self.layout_link.addWidget(button)
        self.links.append(button)

    def add_advertisement(self, url):
        ad_list = AdvertisementsList(url)
        for ad in ad_list.advertisements:
            label_image_link = LabelImageLinkVertical(ad.text, ad.url, ad.image, 150, 150, 10)
            label_image_link.setFocusPolicy(Qt.StrongFocus)
            self.banners_stack.addWidget(label_image_link)

        self.layout_link.addWidget(self.banners_stack, alignment=Qt.AlignCenter)
        advertisement_carousel = ad_list.advertisements_carousel()

        # spin if there more than one advertisement in the list
        if len(ad_list.advertisements) > 1:
            self.banner_timer = QTimer()
            self.banner_timer.setInterval(ad_list.BANNER_TIMEOUT_MS)
            self.banner_timer.timeout.connect(lambda: self.start_carousel(advertisement_carousel))
            self.banner_timer.start()
        # show only one banner
        elif len(ad_list.advertisements) == 1:
            idx, _ = next(advertisement_carousel)
            self.update_banner(idx)

    def update_banner(self, idx):
        self.banners_stack.setCurrentIndex(idx)

    def start_carousel(self, advertisement_carousel):
        idx, _ = next(advertisement_carousel)
        self.sig_update_banner.emit(idx)

    def add_tab(self, text, icon=None):
        """Create the widget that replaces the normal tab content."""
        button = ButtonTab()
        button.setObjectName(text.lower())
        if text == 'Projects':
            text = 'Projects (beta)'
        button.setText(text)
        button.setFocusPolicy(Qt.StrongFocus)

        if icon:
            button.setIcon(icon)

        self.layout_top.addWidget(button)
        self.buttons.append(button)
        index = self.buttons.index(button)
        button.clicked.connect(lambda b=button, i=index: self.refresh(button, index))

    def refresh(self, button=None, index=None):
        """Refresh pressed status of buttons."""
        widths = []
        for b in self.buttons:
            b.setChecked(False)
            b.setProperty('checked', False)
            widths.append(b.width())

        max_width = max(widths)
        for b in self.buttons:
            b.setMinimumWidth(max_width)

        if button:
            button.setChecked(True)
            button.setProperty('checked', True)

        if index is not None:
            self.sig_index_changed.emit(index)
            self.current_index = index


class TabWidget(QWidget):
    """Curstom Tab Widget that includes a more customizable `tabbar`."""

    sig_current_changed = Signal(int)
    sig_url_clicked = Signal(object)

    def __init__(self, *args, **kwargs):
        """Custom Tab Widget that includes a more customizable `tabbar`."""
        super(TabWidget, self).__init__(*args, **kwargs)
        self.frame_sidebar = FrameTabBar()
        self.frame_tab_content = FrameTabBody()
        self.stack = StackBody()
        self.tabbar = TabBar()

        layout_sidebar = QVBoxLayout()
        layout_sidebar.addWidget(self.tabbar)
        self.frame_sidebar.setLayout(layout_sidebar)

        layout_content = QHBoxLayout()
        layout_content.addWidget(self.stack)
        self.frame_tab_content.setLayout(layout_content)

        layout = QHBoxLayout()
        layout.addWidget(self.frame_sidebar)
        layout.addWidget(self.frame_tab_content)

        self.setLayout(layout)
        self.tabbar.sig_index_changed.connect(self.setCurrentIndex)

        self.tabbar.sig_url_clicked.connect(self.sig_url_clicked)
        self.tabbar.sig_update_banner.connect(self.tabbar.update_banner)

    def count(self):
        """Override Qt method."""
        return self.stack.count()

    def widget(self, index):
        """Override Qt method."""
        return self.stack.widget(index)

    def currentWidget(self):
        """Override Qt method."""
        return self.stack.currentWidget()

    def currentIndex(self):
        """Override Qt method."""
        return self.tabbar.current_index

    def setCurrentIndex(self, index):
        """Override Qt method."""
        if self.currentIndex() != index:
            self.tabbar.current_index = index
            self.tabbar.buttons[index].setChecked(True)
            self.tabbar.buttons[index].setFocus()
            self.stack.setCurrentIndex(index)
            self.sig_current_changed.emit(index)

    def currentText(self):
        """Override Qt method."""
        index = self.currentIndex()
        text = ''
        if index:
            button = self.tabbar.buttons[self.currentIndex()]
            if button:
                text = button.text()
        return text

    def addTab(self, widget, icon=None, text=''):
        """Override Qt method."""
        if widget:
            self.tabbar.add_tab(text, icon)
            self.stack.addWidget(widget)
            self.setCurrentIndex(0)
        else:
            raise Exception('tab widget cant be None')

    def add_link(self, text, url=None):
        """Add links to the bottom area of the custom tab bar."""
        self.tabbar.add_link(text, url)

    def add_advertisement(self, url=None):
        self.tabbar.add_advertisement(url)

    def add_social(self, text, url=None):
        """Add social link on bottom of side bar."""
        self.tabbar.add_social(text, url)

    def set_links_header(self, text):
        """Add links header to the bottom of the custom tab bar."""
        self.tabbar.set_links_header(text)

    def refresh(self):
        """Refresh size of buttons."""
        self.tabbar.refresh()


# --- Local testing
# -----------------------------------------------------------------------------
def local_test():  # pragma: no cover
    """Run local tests."""
    from anaconda_navigator.utils.qthelpers import qapplication

    app = qapplication(test_time=3)
    widget = TabWidget()
    widget.addTab(QLabel('HELLO 1'), text='Home', icon=QIcon())
    widget.addTab(QLabel('HELLO 2'), text='Add', icon=QIcon())
    widget.add_link('link 1')
    widget.showMaximized()
    widget.refresh()
    sys.exit(app.exec_())


if __name__ == '__main__':  # pragma: no cover
    local_test()



"""Tests for login dialog."""
import json
from unittest import mock, TestCase
from anaconda_navigator.api.team_edition_api import TeamEditionAPI

TEST_CHANNELS = {
    'total_count': 1,
    'items': [
        {'id': '1df63-4c1-44-af0f-027308381',
         'name': 'testing_channel',
         'description': '',
         'privacy': 'public',
         'artifact_count': 2,
         'download_count': 1,
         'subchannel_count': 0,
         'group_count': 1,
         'mirror_count': 0,
         'filters': {},
         'owners': ['test_user'],
         'created': '2021-02-17T16:05:34.521373+00:00',
         'updated': None}
    ]
}


class TestTeamEditionApi(TestCase):

    def setUp(self):
        self.token = {
            'token': 'iOiJSUzI1NiIsInR5cCIgOiAiSldUIiIiA6ICI2eTRpaVhkWXl3eC0wQUlWeVAyTlA5OWd6bWhtRUVOVmZ2M2',
            'refresh_token': 'eyiIsInR5cIiwia2lkIiA6ICJhOTMxNTYxYi05NDY4LTRhNTAtOTBlZS1iNDAxY',
            'jwt_token': 'eOiJSUzI1NiIsInR5cCIgOiAiSldUIiA6ICI2eTRpaVhkWXl3eC0wQUlWeVAyTlA5OWd6bWhtRUVOV',
            'jwt_token_refresh': 'hbGciOJIUzI1NiIsInR5cCIlkIiA6ICJhOTMxNTYxYi05NDY4LTRhNTAtOTBlZS1iN'
        }
        ate_api_mock = TeamEditionAPI('https://ate.testing.base.mock', mock.Mock())
        ate_api_mock.load_token = mock.Mock(return_value=json.dumps(self.token))
        self.ate_api = ate_api_mock

    @mock.patch('anaconda_navigator.api.team_edition_api.requests')
    def test_get_channels(self, mock_requests):
        mock_response = mock.Mock()
        mock_response.json.return_value = TEST_CHANNELS
        mock_requests.request.return_value = mock_response

        with self.subTest('Bearer token test'):
            self.assertEqual(self.ate_api.get_channels(), TEST_CHANNELS['items'])

        with self.subTest('Bearer token test'):
            self.assertEqual(self.ate_api.get_channels(), TEST_CHANNELS['items'])



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""
Google analytics tracker utilities.

Pages
-----

/home
/community
/environments

/environments/create
/environments/clone
/environments/remove
/login
/about

"""

# yapf: disable

# Standard library imports
from collections import deque
import hashlib
import locale
import platform
import sys

# Third party imports
from qtpy.QtCore import QObject, QThread, QTimer, Signal
from qtpy.QtWidgets import QDesktopWidget

# Local imports
from anaconda_navigator import __version__ as app_version
from anaconda_navigator.api.anaconda_api import AnacondaAPI
from anaconda_navigator.config import CONF
from anaconda_navigator.external.UniversalAnalytics import Tracker
from anaconda_navigator.utils.encoding import ensure_binary


# yapf: enable

try:
    from qtpy.QtCore import __version__ as QT_VERSION
except ImportError:
    from qtpy.QtCore import QT_VERSION_STR as QT_VERSION


class AnalyticsWorker(QObject):
    """Google analytics tracker worker."""

    sig_finished = Signal(object, object, object)

    def __init__(self, method, args, kwargs):
        """Google analytics tracker worker."""
        super(AnalyticsWorker, self).__init__()
        self.method = method
        self.args = args
        self.kwargs = kwargs
        self._is_finished = False

    def is_finished(self):
        """Return True if the worker process has finished, otherwise False."""
        return self._is_finished

    def start(self):
        """Start the worker process."""
        error = None
        output = None
        try:
            output = self.method(*self.args, **self.kwargs)
        except Exception as err:
            error = err

        self.sig_finished.emit(self, output, error)
        self._is_finished = True


class _GATracker(object):
    """Google analytics tracker."""

    TRACKER_ID = 'UA-27761864-8'
    DEFAULT_TIMEOUT = 5

    def __init__(self):
        """Google analytics tracker."""
        # Avoid blocking the UI
        self._queue = deque()
        self._threads = []
        self._workers = []
        self._timer = QTimer()
        self.api = AnacondaAPI()

        self._timer.setInterval(1000)
        self._timer.timeout.connect(self._clean)

        self.setup()

    def setup(self):
        """Setup the tracker initial configuration."""
        # Set tracker config
        try:
            user = self.api.client_user().get('login', '@anonymous')
        except Exception:
            user = '@anonymous'

        domain = self.api.client_domain()
        normalized_user = domain + '/' + user
        hashed_user = hashlib.md5(ensure_binary(normalized_user)).hexdigest()

        self.tracker = Tracker.create(
            self.TRACKER_ID,
            client_id=hashed_user,
            user_id=hashed_user,
        )
        pkg_version = self.api.conda_package_version

        # Set custom variables
        python_version = '.'.join([str(i) for i in sys.version_info[:3]])
        operating_system = ";".join([platform.uname()[0]] + list(platform.uname()[2:]), )
        pyqt_version = (
            pkg_version(pkg='pyqt5', name='root') or pkg_version(pkg='pyqt4', name='root')
            or pkg_version(pkg='pyqt', name='root')
        )
        conda_version = pkg_version(pkg='conda', name='root')

        # Track initial global data events
        self.track_event('application', 'python-version', label=python_version)
        self.track_event('application', 'language', label=self.get_language())
        self.track_event('application', 'screen-resolution', label=self.get_screen_resolution())
        self.track_event('application', 'operating-system', label=operating_system)
        self.track_event('application', 'version', label=app_version)
        self.track_event('application', 'platform', label=self.api.conda_platform())
        self.info = {
            'python': python_version,
            'language': self.get_language(),
            'os': operating_system,
            'version': app_version,
            'platform': self.api.conda_platform(),
            'qt': QT_VERSION,
            'pyqt': pyqt_version,
            'conda': conda_version
        }

    def _clean(self):
        """Check for inactive workers and remove their references."""
        if self._workers:
            for w in self._workers:
                if w.is_finished():
                    self._workers.remove(w)

        if self._threads:
            for t in self._threads:
                if t.isFinished():
                    self._threads.remove(t)
        else:
            self._timer.stop()

    def _start(self):
        """Start the next item in the worker queue."""
        if len(self._queue) == 1:
            thread = self._queue.popleft()
            thread.start()
            self._timer.start()

    def _create_worker(self, method, *args, **kwargs):
        """Create a worker for this client to be run in a separate thread."""
        provide_analytics = CONF.get('main', 'provide_analytics')

        thread = QThread()
        worker = AnalyticsWorker(method, args, kwargs)

        if provide_analytics:
            worker.moveToThread(thread)
            worker.sig_finished.connect(self._start)
            worker.sig_finished.connect(thread.quit)
            thread.started.connect(worker.start)
            self._queue.append(thread)
            self._threads.append(thread)
            self._workers.append(worker)
            self._start()

        return worker

    def track_event(self, category, action, label=None):
        """Track analytyics event."""
        return self._create_worker(
            self.tracker.send,
            'event',
            category=category,
            action=action,
            label=str(label),
            anonymizeIp=True,
        )

    def track_page(self, page, pagetitle=None):
        """
        Track analytyics page.

        Page in the context of Navigator is a tab, a dialog, a message box.
        """
        if pagetitle is None:
            title = ' '.join([i for i in page.split('/') if i])
        else:
            title = pagetitle

        title = title.capitalize()
        return self._create_worker(
            self.tracker.send,
            'pageview',
            page=page,
            pagetitle=title,
            anonymizeIp=True,
        )

    def set_client_id(self, client_id=None):
        """Set the client tracker ID."""
        self.tracker = Tracker.create('UA-74661388-1', client_id=client_id, user_id=client_id)

    @staticmethod
    def get_language():
        """Return the locale language."""
        # Process locale language
        try:
            lang = locale.getdefaultlocale()[0]
        except Exception:
            lang = None

        if not lang:
            lang = 'en'

        return lang

    @staticmethod
    def get_screen_resolution():
        """Return the screen resolution of the primary screen."""
        try:
            widget = QDesktopWidget()
            geometry = widget.availableGeometry(widget.primaryScreen())
            value = "{0}x{1}".format(geometry.width(), geometry.height())
        except Exception:
            value = None
        return value


GA_TRACKER = None


def GATracker():
    """Google Analytics Tracker."""
    global GA_TRACKER

    if GA_TRACKER is None:
        GA_TRACKER = _GATracker()

    return GA_TRACKER


# --- Local testing
# -----------------------------------------------------------------------------
def print_test(worker, output, error):  # pragma: no cover
    """Print test output from worker."""
    print(output, error)


def local_test():  # pragma: no cover
    """Main local test."""
    from anaconda_navigator.utils.qthelpers import qapplication
    app = qapplication()
    tracker = GATracker()
    worker = tracker.track_page('/home')
    worker.sig_finished.connect(print_test)
    print(worker)
    print(tracker.get_screen_resolution())
    print(tracker.get_language())
    app.exec_()


if __name__ == '__main__':  # pragma: no cover
    local_test()



# -*- coding: utf-8 -*-

"""Installable PyCharm application descriptions."""

__all__ = ['PyCharmProApp', 'PyCharmCEApp']

import json
import os
import typing
import webbrowser
from anaconda_navigator import config as navigator_config
from anaconda_navigator.static import images
from . import base
from . import detectors

if typing.TYPE_CHECKING:
    import typing_extensions
    from anaconda_navigator.api import process
    from anaconda_navigator.config import user as user_config


MAC_APPS: typing.Optional[str] = detectors.join(
    detectors.Mac.home, 'Library', 'Application Support', 'JetBrains', 'Toolbox', 'apps',
)
LINUX_APPS: typing.Optional[str] = detectors.join(
    detectors.Linux.home, '.local', 'share', 'JetBrains', 'Toolbox', 'apps',
)
WIN_APPS: typing.Optional[str] = detectors.join(
    detectors.Win.local_app_data, 'JetBrains', 'Toolbox', 'apps',
)


class CheckVersion:
    """
    Detect version of the PyCharm application.

    :param product_code: acceptable product code(s)

                         Known options:

                         - PY: PyCharm Pro
                         - PC: PyCharm Community
                         - PE: PyCharm Edu
    :param configuration_path: Path to the `product-info.json` file relative to the root of the application.
    """

    __slots__ = ('__product_code', '__configuration_path')

    def __init__(
            self,
            product_code: typing.Union[str, typing.Iterable[str]],
            configuration_path: str = 'product-info.json',
    ) -> None:
        """Initialize new :class:`~CheckVersion` instance."""
        if isinstance(product_code, str):
            product_code = (product_code,)
        else:
            product_code = tuple(product_code)

        self.__product_code: 'typing_extensions.Final[typing.Tuple[str, ...]]' = product_code
        self.__configuration_path: 'typing_extensions.Final[str]' = configuration_path

    def __call__(
            self,
            parent: typing.Iterator[detectors.DetectedApplication],
    ) -> typing.Iterator[detectors.DetectedApplication]:
        """Iterate through detected applications."""
        application: detectors.DetectedApplication
        for application in parent:
            if not application.root:
                continue

            product_code: str
            version: str
            try:
                stream: typing.TextIO
                with open(os.path.join(application.root, self.__configuration_path), 'rt', encoding='utf-8') as stream:
                    data: typing.Mapping[str, typing.Any] = json.load(stream)

                product_code = data['productCode']
                version = data['version']
            except (OSError, ValueError, TypeError, AttributeError):
                continue

            if product_code in self.__product_code:
                yield application.replace(version=version)


class BasePyCharmApp(base.BaseInstallableApp):
    """Common parts for PyCharm applications."""

    def __init__(
            self,
            app_name: str,
            display_name: str,
            description: str,
            image_path: str,
            detector: 'detectors.Source',
            process_api: 'process.WorkerManager',
            config: 'user_config.UserConfig',
    ) -> None:
        """Initialize new :class:`~BasePyCharmApp` instance."""
        super().__init__(
            app_name=app_name,
            display_name=display_name,
            description=description,
            image_path=image_path,
            detector=detector,
            is_available=navigator_config.BITS_64,
            needs_license=False,
            process_api=process_api,
            config=config,
        )

    def install_extensions(self) -> 'process.ProcessWorker':
        """Install app extensions."""
        return self._process_api.create_process_worker(['python', '--version'])

    def update_config(self, prefix: str) -> None:
        """Update user config to use selected Python prefix interpreter."""


class PyCharmProApp(BasePyCharmApp):
    """PyCharm Professional application."""

    def __init__(self, process_api: 'process.WorkerManager', config: 'user_config.UserConfig') -> None:
        """Initialize new :class:`~PyCharmProApp` instance."""
        detector: 'typing_extensions.Final[detectors.Source]' = detectors.Group(
            detectors.Group(
                detectors.CheckConfiguredRoots('pycharm_pro_path', configuration=config),
                detectors.mac_only(),

                detectors.check_known_mac_roots('PyCharm.app'),
                detectors.Group(
                    detectors.CheckKnownRoots(detectors.join(MAC_APPS, 'PyCharm-P')),
                    detectors.StepIntoRoot(starts_with='ch-'),
                    detectors.StepIntoRoot(reverse=True),
                    detectors.StepIntoRoot(equals='PyCharm.app'),
                ),

                detectors.AppendExecutable(
                    os.path.join('Contents', 'MacOS', 'pycharm'),
                ),

                CheckVersion(
                    product_code='PY',
                    configuration_path=os.path.join('Contents', 'Resources', 'product-info.json'),
                ),
            ),
            detectors.Group(
                detectors.CheckConfiguredRoots('pycharm_pro_path', configuration=config),
                detectors.linux_only(),

                detectors.Group(
                    detectors.CheckKnownRoots(
                        detectors.join(detectors.Linux.root, 'opt'),
                    ),
                    detectors.StepIntoRoot(starts_with='pycharm', reverse=True),
                ),
                detectors.Group(
                    detectors.CheckKnownRoots(
                        detectors.join(LINUX_APPS, 'PyCharm-P'),
                    ),
                    detectors.StepIntoRoot(starts_with='ch-'),
                    detectors.StepIntoRoot(reverse=True),
                ),
                detectors.CheckKnownRoots(
                    detectors.join(detectors.Linux.root, 'snap', 'pycharm-professional', 'current'),
                    detectors.join(
                        detectors.Linux.root, 'var', 'lib', 'snapd', 'snap', 'pycharm-professional', 'current'
                    ),
                ),

                detectors.AppendExecutable(os.path.join('bin', 'pycharm.sh')),

                CheckVersion(
                    product_code='PY',
                ),
            ),
            detectors.Group(
                detectors.CheckConfiguredRoots('pycharm_pro_path', configuration=config),
                detectors.win_only(),

                detectors.Group(
                    detectors.CheckKnownRoots(
                        detectors.join(detectors.Win.program_files_x64, 'JetBrains'),
                        detectors.join(detectors.Win.program_files_x86, 'JetBrains'),
                    ),
                    detectors.StepIntoRoot(starts_with='PyCharm', reverse=True),
                ),
                detectors.Group(
                    detectors.CheckKnownRoots(
                        detectors.join(WIN_APPS, 'PyCharm-P'),
                    ),
                    detectors.StepIntoRoot(starts_with='ch-'),
                    detectors.StepIntoRoot(reverse=True),
                ),

                detectors.AppendExecutable(
                    os.path.join('bin', 'pycharm64.exe'),
                    os.path.join('bin', 'pycharm32.exe'),
                    os.path.join('bin', 'pycharm.exe'),
                ),

                CheckVersion(
                    product_code='PY',
                ),
            ),
        )

        super().__init__(
            app_name='pycharm_pro',
            display_name='PyCharm Professional',
            description=(
                'A full-fledged IDE by JetBrains for both Scientific and Web Python development. Supports HTML, JS, '
                'and SQL.'
            ),
            image_path=images.PYCHARM_ICON_1024_PATH,
            detector=detector,
            process_api=process_api,
            config=config,
        )

    def install(self) -> None:
        """Install application."""
        webbrowser.open_new_tab('https://www.anaconda.com/pycharm_navigator')


class PyCharmCEApp(BasePyCharmApp):
    """PyCharm Community Edition application."""

    def __init__(self, process_api: 'process.WorkerManager', config: 'user_config.UserConfig') -> None:
        """Initialize new :class:`~PyCharmCEApp` instance."""
        detector: 'typing_extensions.Final[detectors.Source]' = detectors.Group(
            detectors.Group(
                detectors.CheckConfiguredRoots('pycharm_pro_path', configuration=config),
                detectors.mac_only(),

                detectors.check_known_mac_roots('PyCharm CE.app', 'PyCharm Edu.app'),
                detectors.Group(
                    detectors.CheckKnownRoots(
                        detectors.join(MAC_APPS, 'PyCharm-C'),
                        detectors.join(MAC_APPS, 'PyCharm-E'),
                    ),
                    detectors.StepIntoRoot(starts_with='ch-'),
                    detectors.StepIntoRoot(reverse=True),
                    detectors.StepIntoRoot(equals=('PyCharm CE.app', 'PyCharm Edu.app')),
                ),

                detectors.AppendExecutable(
                    os.path.join('Contents', 'MacOS', 'pycharm'),
                ),

                CheckVersion(
                    product_code=('PC', 'PE'),
                    configuration_path=os.path.join('Contents', 'Resources', 'product-info.json'),
                ),
            ),
            detectors.Group(
                detectors.CheckConfiguredRoots('pycharm_pro_path', configuration=config),
                detectors.linux_only(),

                detectors.Group(
                    detectors.CheckKnownRoots(
                        detectors.join(detectors.Linux.root, 'opt'),
                    ),
                    detectors.StepIntoRoot(starts_with='pycharm', reverse=True),
                ),
                detectors.Group(
                    detectors.CheckKnownRoots(
                        detectors.join(LINUX_APPS, 'PyCharm-C'),
                        detectors.join(LINUX_APPS, 'PyCharm-E'),
                    ),
                    detectors.StepIntoRoot(starts_with='ch-'),
                    detectors.StepIntoRoot(reverse=True),
                ),
                detectors.CheckKnownRoots(
                    detectors.join(detectors.Linux.root, 'snap', 'pycharm-community', 'current'),
                    detectors.join(
                        detectors.Linux.root, 'var', 'lib', 'snapd', 'snap', 'pycharm-community', 'current'
                    ),
                    detectors.join(detectors.Linux.root, 'snap', 'pycharm-educational', 'current'),
                    detectors.join(
                        detectors.Linux.root, 'var', 'lib', 'snapd', 'snap', 'pycharm-educational', 'current'
                    ),
                ),

                detectors.AppendExecutable(os.path.join('bin', 'pycharm.sh')),

                CheckVersion(
                    product_code=('PC', 'PE'),
                ),
            ),
            detectors.Group(
                detectors.CheckConfiguredRoots('pycharm_pro_path', configuration=config),
                detectors.win_only(),

                detectors.Group(
                    detectors.CheckKnownRoots(
                        detectors.join(detectors.Win.program_files_x64, 'JetBrains'),
                        detectors.join(detectors.Win.program_files_x86, 'JetBrains'),
                    ),
                    detectors.StepIntoRoot(starts_with='PyCharm', reverse=True),
                ),
                detectors.Group(
                    detectors.CheckKnownRoots(
                        detectors.join(WIN_APPS, 'PyCharm-C'),
                        detectors.join(WIN_APPS, 'PyCharm-E'),
                    ),
                    detectors.StepIntoRoot(starts_with='ch-'),
                    detectors.StepIntoRoot(reverse=True),
                ),

                detectors.AppendExecutable(
                    os.path.join('bin', 'pycharm64.exe'),
                    os.path.join('bin', 'pycharm32.exe'),
                    os.path.join('bin', 'pycharm.exe'),
                ),

                CheckVersion(
                    product_code=('PC', 'PE'),
                ),
            ),
        )

        super().__init__(
            app_name='pycharm_ce',
            display_name='PyCharm Community',
            description=(
                'An IDE by JetBrains for pure Python development. Supports code completion, listing, and debugging.'
            ),
            image_path=images.PYCHARM_CE_ICON_1024_PATH,
            detector=detector,
            process_api=process_api,
            config=config,
        )
